{
  description = "Michael Thomas's NixOS configuration";

  inputs = {
    nixpkgs.url = "nixpkgs/nixos-24.05";
    unstable.url = "nixpkgs/nixos-unstable";
    nur.url = "github:nix-community/NUR";

    home-manager = {
      url = "github:rycee/home-manager/release-24.05";
      inputs = {
        nixpkgs.follows = "nixpkgs";
      };
    };
    darwin = {
      url = "github:lnl7/nix-darwin/master";
      inputs.nixpkgs.follows = "nixpkgs";
    };

    utils.url = "github:numtide/flake-utils";
    flake-parts.url = "github:hercules-ci/flake-parts";
    treefmt-nix.url = "github:numtide/treefmt-nix";
    nixvim = {
      url = "github:nix-community/nixvim/nixos-24.05";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    rust-overlay.url = "github:oxalica/rust-overlay";
    nix-vscode-extensions.url = "github:nix-community/nix-vscode-extensions";
    hyprland.url = "git+https://github.com/hyprwm/Hyprland?submodules=1";
    ags.url = "github:Aylur/ags";
    agenix = {
      url = "github:ryantm/agenix";
      inputs.nixpkgs.follows = "nixpkgs";
    };
  };

  outputs = inputs @ {
    self,
    darwin,
    home-manager,
    flake-parts,
    ...
  }:
    flake-parts.lib.mkFlake {inherit inputs;} {
      flake = let
        baseHomeModule = {
          home-manager.useGlobalPkgs = true;
          home-manager.useUserPackages = true;
          home-manager.extraSpecialArgs = {inherit inputs;};
        };
      in {
        overlays = import ./overlays {inherit inputs;};

        darwinConfigurations = {
          mac = darwin.lib.darwinSystem {
            system = "aarch64-darwin";
            specialArgs = {inherit inputs;};
            modules = [
              self.overlaysModule
              home-manager.darwinModules.home-manager
              baseHomeModule
              {
                home-manager.users.michael = import ./user/environments/mac/home.nix;
              }

              ./modules/yabai.nix
              ./machines/mac/configuration.nix
            ];
          };
        };

        homeConfigurations = {
          nixos = inputs.home-manager.lib.homeManagerConfiguration {
            system = "x86_64-linux";
            homeDirectory = "/home/michael";
            username = "michael";
            stateVersion = "21.05";
            configuration = {
              config,
              pkgs,
              ...
            }: {
              programs.home-manager.enable = true;

              imports = [
                ./user/environments/nixos/home.nix
              ];
            };
          };

          mac = inputs.home-manager.lib.homeManagerConfiguration {
            system = "aarch64-darwin";
            homeDirectory = "/Users/michael";
            username = "michael";
            stateVersion = "22.05";
            configuration = {
              config,
              pkgs,
              ...
            }: {
              programs.home-manager.enable = true;

              imports = [
                ./user/environments/mac/home.nix
              ];
            };
          };
        };
      };
      systems = [
        "aarch64-darwin"
        "aarch64-linux"
        "x86_64-linux"
      ];
      imports = [./flakes];
    };
}
